<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Report\ReportService;
use Teknisa\MAN\Services;
use Zeedhi\Framework\DTO;
use Teknisa\MAN\Utilities;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;


class RelFinanceiro{
    
    const PARAM_P_TELA = 'P_';
    const PARAM_PT_TELA = 'P_T';
    protected $relatorioService;
    protected $relMapaService;
    protected $util;
    protected $birtReport;
    protected $inserirParams = array(
        'NOMEINI'=> null,
        'NOMEFIM' => null
    );
    
    public function __construct (Utilities\Util $util,ReportService $reportService, Services\Relatorio $relatorioService, 
    BirtReport $birtReport) {
        $this->reportService = $reportService;
        $this->relatorioService = $relatorioService;
        $this->util = $util;
        $this->birtReport = $birtReport;
    }
    
    public function gerarRelatorio(DTO\Request\Row $request, DTO\Response $response){
        $conditions = $this->util->getParamsAsArray($request);
        $conditions['CDPICTPROD'] = str_replace(array('*',9),'@',$conditions['CDPICTPROD']);
        $conditions = $this->buildSubstutirParams($conditions);
        $conditions = $this->buildPeriodo($conditions);
        $conditions = $this->inserirParams($conditions);
        $conditions['__report_name'] = $this->getNameReport($conditions['REPORTVALUE']);
        $params = $this->createParamsArray($conditions);
        $params = array(
            'ParametersRow' => $params,
            'ReportName'    => $conditions['__report_name']
        );
        $this->getRelatorio($params,$response);
    }
    
    private function buildSubstutirParams($conditions){
        if(isset($conditions['CDLOJA']) && (\Zeedhi\Framework\Util\Functions::arrayKeyExists('CDLOJA', $conditions) || count($conditions['CDLOJA']))) {
            $conditions['CDLOJA'] = $conditions['FILILOJA'];
        }
        return $conditions;
    }
    
    private function parameterMapping($conditions){
        $parameterMapping = array();
        foreach ($conditions as $key=>$value) {
            $parameterMapping[$key] = $key;
        }
        unset($parameterMapping['__report_name']);
        return $parameterMapping;
    }
    
    public function getRelatorio ($relatorioData , DTO\Response $response){
         try {
            $birtURL = $this->reportService->createRemoteReport('BIRT', $relatorioData['ReportName'], $relatorioData['ParametersRow'], true);
            $response->addDataset(new DataSet('URL',array('url'=>$birtURL)));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage() , 500));
        }
     }
     
    private function insereP ($text){
        return self::PARAM_P_TELA.$text;
    }
    
    private function insereP_T ($text){
        return self::PARAM_PT_TELA.$text;
    }
    
    private function getNameReport($name){
        if($name=='rfc'){
            return 'relFinanceiro';
        }
        else{
            return 'relMargemPonderada';
        }
        
    }

    private function buildPeriodo ($conditions){
        if(\Zeedhi\Framework\Util\Functions::arrayKeyExists('DTENTRVENDA', $conditions)) {
            $conditions[$this->insereP('DTLANCINI')] = $conditions['DTENTRVENDA']['start'];
            $conditions[$this->insereP('DTLANCFIN')] = $conditions['DTENTRVENDA']['end'];
        }
        return $conditions;
    }
    
    private function inserirParams($conditions){
        foreach ($this->inserirParams as $key =>  $value) {
            $conditions[$this->insereP($key)] = $value;
        }
        return $conditions;
    }
    
    private function createParamsArray($conditions){
        $params = array (
            "NRORG"         => $conditions["NRORG"],
            "CDPICTPROD"    => $conditions["CDPICTPROD"],
            "P_CDFILIAL"    => !empty($conditions["CDFILIAL"]) ? implode('_', $conditions["CDFILIAL"]) : '',
            "P_TCDFILIAL"   => !empty($conditions["CDFILIAL"]) ? '' : 'T',
            "P_CDCAIXA"     => !empty($conditions["CDCAIXA"]) ? implode('_', $conditions["CDCAIXA"]) : '',
            "P_TCDCAIXA"    => !empty($conditions["CDCAIXA"]) ? '' : 'T',
            "P_CDLOJA"      => !empty($conditions["CDLOJA"]) ? implode('_', $conditions["CDLOJA"]) : '',
            "P_TCDLOJA"     => !empty($conditions["CDLOJA"]) ? '' : 'T',
            
            "P_TIPOCUSTO"   => isset($conditions["CDTIPOCUSTO"]) ? $conditions["CDTIPOCUSTO"]: null,
            "P_NIVEL"       => isset($conditions["CDPRODUTONIVEL"]) ? $conditions["CDPRODUTONIVEL"]: null,
            "P_ORDERVALUE"  => isset($conditions["ORDENVALUE"]) ? $conditions["ORDENVALUE"]: null,
            "P_DATAINI"     => $conditions["P_DTLANCINI"],
            "P_DATAFIN"     => $conditions["P_DTLANCFIN"]
        );
        return $params;
    }
}